@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,6 +14,8 @@
 #include <dhcpsrv/d2_client_mgr.h>
 #include <asiolink/io_address.h>
 #include <stats/stats_mgr.h>
+#include <set>
+#include <vector>
 
 using namespace isc;
 using namespace isc::asiolink;
@@ -60,6 +62,17 @@ namespace {
 ///   - The reservations-in-subnet and reservations-out-of-pool flags are set to
 ///     true to test that only out of pool reservations are honored.
 ///
+/// - Configuration 5:
+///   - Selects random allocator for addresses.
+///   - One subnet with three distinct pools.
+///   - Random allocator enabled globally for addresses.
+///   - Iterative allocator for prefix delegation.
+///
+/// - Configuration 6:
+///   - Selects random allocator for delegated prefixes.
+///   - One subnet with three distinct pools.
+///   - Random allocator enabled globally for delegated prefixes.
+///   - Iterative allocator for address allocation.
 const char* CONFIGS[] = {
     // Configuration 0
     "{ \"interfaces-config\": {"
@@ -214,7 +227,65 @@ const char* CONFIGS[] = {
         "       }"
         "    ]"
         "} ]"
-    "}"
+    "}",
+
+    // Configuration 5
+    "{ \"interfaces-config\": {"
+        "  \"interfaces\": [ \"*\" ]"
+        "},"
+        "\"allocator\": \"random\","
+        "\"pd-allocator\": \"iterative\","
+        "\"preferred-lifetime\": 3000,"
+        "\"rebind-timer\": 2000, "
+        "\"renew-timer\": 1000, "
+        "\"subnet6\": ["
+        "    {"
+        "        \"pools\": ["
+        "            {"
+        "                \"pool\": \"3000::20 - 3000::60\""
+        "            }"
+        "        ],"
+        "        \"pd-pools\": ["
+        "            {"
+        "                \"prefix\": \"2001:db8:3::\", "
+        "                \"prefix-len\": 48, "
+        "                \"delegated-len\": 64"
+        "            }"
+        "        ],"
+        "        \"subnet\": \"3000::/32\", "
+        "        \"interface\": \"eth0\""
+        "    }"
+        "],"
+        "\"valid-lifetime\": 4000 }",
+
+    // Configuration 6
+    "{ \"interfaces-config\": {"
+        "  \"interfaces\": [ \"*\" ]"
+        "},"
+        "\"allocator\": \"iterative\","
+        "\"pd-allocator\": \"random\","
+        "\"preferred-lifetime\": 3000,"
+        "\"rebind-timer\": 2000, "
+        "\"renew-timer\": 1000, "
+        "\"subnet6\": ["
+        "    {"
+        "        \"pools\": ["
+        "            {"
+        "                \"pool\": \"3000::20 - 3000::60\""
+        "            }"
+        "        ],"
+        "        \"pd-pools\": ["
+        "            {"
+        "                \"prefix\": \"2001:db8:3::\", "
+        "                \"prefix-len\": 48, "
+        "                \"delegated-len\": 64"
+        "            }"
+        "        ],"
+        "        \"subnet\": \"3000::/32\", "
+        "        \"interface\": \"eth0\""
+        "    }"
+        "],"
+        "\"valid-lifetime\": 4000 }",
 };
 
 /// @brief Test fixture class for testing 4-way exchange: Solicit-Advertise,
@@ -242,17 +313,86 @@ public:
         isc::stats::StatsMgr::instance().removeAll();
     }
 
+    /// @brief Check that server processes correctly a prefix hint sent by the
+    /// client. This test checks that the server doesn't allocate colliding
+    /// prefixes as a result of receiving hints from two clients which set the
+    /// non-significant bytes of the prefix in their hints. The server should
+    /// zero the non-significant bytes of the hint and allocate the prefix of
+    /// the correct (configured) length.
+    void directClientPrefixHint();
+
+    /// @brief Check that the server assigns a delegated prefix that is later
+    /// returned to the client for various prefix hints.
+    ///
+    /// When the client renews the lease, it sends a prefix hint with the same
+    /// prefix but with a different prefix length. In another case, the client
+    /// asks for the same prefix length but different prefix. In both cases,
+    /// the server should return an existing lease.
+    void directClientPrefixLengthHintRenewal();
+
+    /// @brief This test verifies that the same options can be specified on the
+    /// global level, subnet level and pool level. The options associated with
+    /// pools are used when the lease is handed out from these pools.
+    void optionsInheritance();
+
+    /// @brief This test verifies that it is possible to specify an excluded
+    /// prefix (RFC 6603) and send it back to the client requesting prefix
+    /// delegation.
+    void directClientExcludedPrefix();
+
+    /// @brief Check that when the client includes the Rapid Commit option in
+    /// its Solicit, the server responds with Reply and commits the lease.
+    void rapidCommitEnable();
+
+    /// @brief Check that the server responds with Advertise if the client
+    /// hasn't included the Rapid Commit option in the Solicit.
+    void rapidCommitNoOption();
+
+    /// @brief Check that when the Rapid Commit support is disabled for the
+    /// subnet the server replies with an Advertise and ignores the Rapid Commit
+    /// option sent by the client.
+    void rapidCommitDisable();
+
+    /// @brief This test verifies that regular Solicit/Adv/Request/Reply
+    /// exchange will result in appropriately set statistics.
+    void sarrStats();
+
+    /// @brief This test verifies that pkt6-receive-drop is increased properly
+    /// when the client's packet is rejected due to mismatched server-id value.
+    void pkt6ReceiveDropStat1();
+
+    /// @brief This test verifies that pkt6-receive-drop is increased properly
+    /// when the client's packet is rejected due to being unicast communication.
+    void pkt6ReceiveDropStat2();
+
+    /// @brief This test verifies that pkt6-receive-drop is increased properly
+    /// when the client's packet is rejected due to having too many client-id
+    /// options (exactly one is expected).
+    void pkt6ReceiveDropStat3();
+
+    /// @brief This test verifies that in pool reservations are ignored when the
+    /// reservations-out-of-pool flag is set to true.
+    void reservationModeOutOfPool();
+
+    /// @brief This test verifies that the in-pool reservation can be assigned
+    /// to a client not owning this reservation when the
+    /// reservations-out-of-pool flag is set to true.
+    void reservationIgnoredInOutOfPoolMode();
+
+    /// @brief This test verifies that random allocator is used according
+    /// to the configuration and it allocates random addresses.
+    void randomAddressAllocation();
+
+    /// @brief This test verifies that random allocator is used according
+    /// to the configuration and it allocates random prefixes.
+    void randomPrefixAllocation();
+
     /// @brief Interface Manager's fake configuration control.
     IfaceMgrTestConfig iface_mgr_test_config_;
 };
 
-/// Check that server processes correctly a prefix hint sent by the client.
-/// This test checks that the server doesn't allocate colliding prefixes
-/// as a result of receiving hints from two clients which set the
-/// non-significant bytes of the prefix in their hints. The server should zero
-/// the non-significant bytes of the hint and allocate the prefix of the
-/// correct (configured) length.
-TEST_F(SARRTest, directClientPrefixHint) {
+void
+SARRTest::directClientPrefixHint() {
     Dhcp6Client client;
     // Configure client to request IA_PD.
     client.requestPrefix();
@@ -306,10 +446,76 @@ TEST_F(SARRTest, directClientPrefixHint) {
     ASSERT_TRUE(lease_server);
 }
 
-// This test verifies that the same options can be specified on the global
-// level, subnet level and pool level. The options associated with pools
-// are used when the lease is handed out from these pools.
-TEST_F(SARRTest, optionsInheritance) {
+TEST_F(SARRTest, directClientPrefixHint) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    directClientPrefixHint();
+}
+
+TEST_F(SARRTest, directClientPrefixHintMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    directClientPrefixHint();
+}
+
+void
+SARRTest::directClientPrefixLengthHintRenewal() {
+    Dhcp6Client client;
+    // Configure client to request IA_PD.
+    client.requestPrefix();
+    configure(CONFIGS[0], *client.getServer());
+    // Make sure we ended-up having expected number of subnets configured.
+    const Subnet6Collection* subnets = CfgMgr::instance().getCurrentCfg()->
+        getCfgSubnets6()->getAll();
+    ASSERT_EQ(1, subnets->size());
+    // Append IAPREFIX option to the client's message.
+    ASSERT_NO_THROW(client.requestPrefix(5678, 64, asiolink::IOAddress("2001:db8:3:36::")));
+    // Perform 4-way exchange.
+    ASSERT_NO_THROW(client.doSARR());
+    // Server should have assigned a prefix.
+    ASSERT_EQ(1, client.getLeaseNum());
+    Lease6 lease_client = client.getLease(0);
+    // The server should respect the prefix hint.
+    EXPECT_EQ("2001:db8:3:36::", lease_client.addr_.toText());
+    // Server ignores other parts of the IAPREFIX option.
+    EXPECT_EQ(64, lease_client.prefixlen_);
+    EXPECT_EQ(3000, lease_client.preferred_lft_);
+    EXPECT_EQ(4000, lease_client.valid_lft_);
+    Lease6Ptr lease_server = checkLease(lease_client);
+    // Check that the server recorded the lease.
+    ASSERT_TRUE(lease_server);
+
+    // Request the same prefix with a different length. The server should
+    // return an existing lease.
+    client.clearRequestedIAs();
+    ASSERT_NO_THROW(client.requestPrefix(5678, 80, IOAddress("2001:db8:3:36::")));
+    ASSERT_NO_THROW(client.doSARR());
+    ASSERT_EQ(1, client.getLeaseNum());
+    lease_client = client.getLease(0);
+    EXPECT_EQ("2001:db8:3:36::", lease_client.addr_.toText());
+    EXPECT_EQ(64, lease_client.prefixlen_);
+
+    // Try to request another prefix. The client should still get the existing
+    // lease.
+    client.clearRequestedIAs();
+    ASSERT_NO_THROW(client.requestPrefix(5678, 64, IOAddress("2001:db8:3:37::")));
+    ASSERT_NO_THROW(client.doSARR());
+    ASSERT_EQ(1, client.getLeaseNum());
+    lease_client = client.getLease(0);
+    EXPECT_EQ("2001:db8:3:36::", lease_client.addr_.toText());
+    EXPECT_EQ(64, lease_client.prefixlen_);
+}
+
+TEST_F(SARRTest, directClientPrefixLengthHintRenewal) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    directClientPrefixLengthHintRenewal();
+}
+
+TEST_F(SARRTest, directClientPrefixLengthHintRenewalMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    directClientPrefixLengthHintRenewal();
+}
+
+void
+SARRTest::optionsInheritance() {
     Dhcp6Client client;
     // Request a single address and single prefix.
     ASSERT_NO_THROW(client.requestPrefix(0xabac, 64, IOAddress("2001:db8:4::")));
@@ -367,9 +573,18 @@ TEST_F(SARRTest, optionsInheritance) {
     ASSERT_TRUE(client.hasOptionWithAddress(D6O_SNTP_SERVERS, "3000:2::2"));
 }
 
-/// This test verifies that it is possible to specify an excluded prefix
-/// (RFC 6603) and send it back to the client requesting prefix delegation.
-TEST_F(SARRTest, directClientExcludedPrefix) {
+TEST_F(SARRTest, optionsInheritance) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    optionsInheritance();
+}
+
+TEST_F(SARRTest, optionsInheritanceMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    optionsInheritance();
+}
+
+void
+SARRTest::directClientExcludedPrefix() {
     Dhcp6Client client;
     // Configure client to request IA_PD.
     client.requestPrefix();
@@ -408,9 +623,18 @@ TEST_F(SARRTest, directClientExcludedPrefix) {
     EXPECT_EQ(120, static_cast<unsigned>(pd_exclude->getExcludedPrefixLength()));
 }
 
-// Check that when the client includes the Rapid Commit option in its
-// Solicit, the server responds with Reply and commits the lease.
-TEST_F(SARRTest, rapidCommitEnable) {
+TEST_F(SARRTest, directClientExcludedPrefix) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    directClientExcludedPrefix();
+}
+
+TEST_F(SARRTest, directClientExcludedPrefixMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    directClientExcludedPrefix();
+}
+
+void
+SARRTest::rapidCommitEnable() {
     Dhcp6Client client;
     // Configure client to request IA_NA
     client.requestAddress();
@@ -446,9 +670,18 @@ TEST_F(SARRTest, rapidCommitEnable) {
     EXPECT_EQ(1, CfgMgr::instance().getD2ClientMgr().getQueueSize());
 }
 
-// Check that the server responds with Advertise if the client hasn't
-// included the Rapid Commit option in the Solicit.
-TEST_F(SARRTest, rapidCommitNoOption) {
+TEST_F(SARRTest, rapidCommitEnable) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    rapidCommitEnable();
+}
+
+TEST_F(SARRTest, rapidCommitEnableMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    rapidCommitEnable();
+}
+
+void
+SARRTest::rapidCommitNoOption() {
     Dhcp6Client client;
     // Configure client to request IA_NA
     client.requestAddress();
@@ -476,10 +709,18 @@ TEST_F(SARRTest, rapidCommitNoOption) {
     EXPECT_EQ(0, CfgMgr::instance().getD2ClientMgr().getQueueSize());
 }
 
-// Check that when the Rapid Commit support is disabled for the subnet
-// the server replies with an Advertise and ignores the Rapid Commit
-// option sent by the client.
-TEST_F(SARRTest, rapidCommitDisable) {
+TEST_F(SARRTest, rapidCommitNoOption) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    rapidCommitNoOption();
+}
+
+TEST_F(SARRTest, rapidCommitNoOptionMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    rapidCommitNoOption();
+}
+
+void
+SARRTest::rapidCommitDisable() {
     Dhcp6Client client;
     // The subnet assigned to eth1 has Rapid Commit disabled.
     client.setInterface("eth1");
@@ -511,9 +752,18 @@ TEST_F(SARRTest, rapidCommitDisable) {
     EXPECT_EQ(0, CfgMgr::instance().getD2ClientMgr().getQueueSize());
 }
 
-// This test verifies that regular Solicit/Adv/Request/Reply exchange will
-// result in appropriately set statistics.
-TEST_F(SARRTest, sarrStats) {
+TEST_F(SARRTest, rapidCommitDisable) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    rapidCommitDisable();
+}
+
+TEST_F(SARRTest, rapidCommitDisableMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    rapidCommitDisable();
+}
+
+void
+SARRTest::sarrStats() {
 
     // Let's use one of the existing configurations and tell the client to
     // ask for an address.
@@ -577,9 +827,18 @@ TEST_F(SARRTest, sarrStats) {
     EXPECT_EQ(2, pkt6_sent->getInteger().first);
 }
 
-// This test verifies that pkt6-receive-drop is increased properly when the
-// client's packet is rejected due to mismatched server-id value.
-TEST_F(SARRTest, pkt6ReceiveDropStat1) {
+TEST_F(SARRTest, sarrStats) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    sarrStats();
+}
+
+TEST_F(SARRTest, sarrStatsMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    sarrStats();
+}
+
+void
+SARRTest::pkt6ReceiveDropStat1() {
 
     // Dummy server-id (0xff repeated 10 times)
     std::vector<uint8_t> data(10, 0xff);
@@ -606,9 +865,18 @@ TEST_F(SARRTest, pkt6ReceiveDropStat1) {
     EXPECT_EQ(1, pkt6_recv_drop->getInteger().first);
 }
 
-// This test verifies that pkt6-receive-drop is increased properly when the
-// client's packet is rejected due to being unicast communication.
-TEST_F(SARRTest, pkt6ReceiveDropStat2) {
+TEST_F(SARRTest, pkt6ReceiveDropStat1) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    pkt6ReceiveDropStat1();
+}
+
+TEST_F(SARRTest, pkt6ReceiveDropStat1MultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    pkt6ReceiveDropStat1();
+}
+
+void
+SARRTest::pkt6ReceiveDropStat2() {
 
     // Let's use one of the existing configurations and tell the client to
     // ask for an address.
@@ -630,10 +898,18 @@ TEST_F(SARRTest, pkt6ReceiveDropStat2) {
     EXPECT_EQ(1, pkt6_recv_drop->getInteger().first);
 }
 
-// This test verifies that pkt6-receive-drop is increased properly when the
-// client's packet is rejected due to having too many client-id options
-// (exactly one is expected).
-TEST_F(SARRTest, pkt6ReceiveDropStat3) {
+TEST_F(SARRTest, pkt6ReceiveDropStat2) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    pkt6ReceiveDropStat2();
+}
+
+TEST_F(SARRTest, pkt6ReceiveDropStat2MultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    pkt6ReceiveDropStat2();
+}
+
+void
+SARRTest::pkt6ReceiveDropStat3() {
 
     // Let's use one of the existing configurations and tell the client to
     // ask for an address.
@@ -658,9 +934,18 @@ TEST_F(SARRTest, pkt6ReceiveDropStat3) {
     EXPECT_EQ(1, pkt6_recv_drop->getInteger().first);
 }
 
-// This test verifies that in pool reservations are ignored when the
-// reservations-out-of-pool flag is set to true.
-TEST_F(SARRTest, reservationModeOutOfPool) {
+TEST_F(SARRTest, pkt6ReceiveDropStat3) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    pkt6ReceiveDropStat3();
+}
+
+TEST_F(SARRTest, pkt6ReceiveDropStat3MultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    pkt6ReceiveDropStat3();
+}
+
+void
+SARRTest::reservationModeOutOfPool() {
     // Create the first client for which we have a reservation out of the
     // dynamic pool.
     Dhcp6Client client;
@@ -694,10 +979,18 @@ TEST_F(SARRTest, reservationModeOutOfPool) {
     ASSERT_EQ("2001:db8:1::3", lease.addr_.toText());
 }
 
-// This test verifies that the in-pool reservation can be assigned to a client
-// not owning this reservation when the reservations-out-of-pool flag is set to
-// true.
-TEST_F(SARRTest, reservationIgnoredInOutOfPoolMode) {
+TEST_F(SARRTest, reservationModeOutOfPool) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    reservationModeOutOfPool();
+}
+
+TEST_F(SARRTest, reservationModeOutOfPoolMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    reservationModeOutOfPool();
+}
+
+void
+SARRTest::reservationIgnoredInOutOfPoolMode() {
     // Create the first client for which we have a reservation out of the
     // dynamic pool.
     Dhcp6Client client;
@@ -716,4 +1009,155 @@ TEST_F(SARRTest, reservationIgnoredInOutOfPoolMode) {
     ASSERT_EQ("2001:db8:1::5", lease.addr_.toText());
 }
 
+TEST_F(SARRTest, reservationIgnoredInOutOfPoolMode) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    reservationIgnoredInOutOfPoolMode();
+}
+
+TEST_F(SARRTest, reservationIgnoredInOutOfPoolModeMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    reservationIgnoredInOutOfPoolMode();
+}
+
+void
+SARRTest::randomAddressAllocation() {
+    // Create the base client and server configuration.
+    Dhcp6Client client;
+    configure(CONFIGS[5], *client.getServer());
+
+    // Record what addresses have been allocated and in what order.
+    std::set<std::string> allocated_na_set;
+    std::vector<IOAddress> allocated_na_vector;
+    std::set<std::string> allocated_pd_set;
+    std::vector<IOAddress> allocated_pd_vector;
+    // Simulate allocations from different clients.
+    for (auto i = 0; i < 30; ++i) {
+        // Create a client from the base client.
+        Dhcp6Client next_client(client.getServer());
+        next_client.requestAddress();
+        next_client.requestPrefix();
+        // Run 4-way exchange.
+        ASSERT_NO_THROW(next_client.doSARR());
+        // We should have one IA_NA and one IA_PD.
+        auto leases_na = next_client.getLeasesByType(Lease::TYPE_NA);
+        ASSERT_EQ(1, leases_na.size());
+        auto leases_pd = next_client.getLeasesByType(Lease::TYPE_PD);
+        ASSERT_EQ(1, leases_pd.size());
+        // Remember allocated address and delegated prefix uniqueness
+        // and order.
+        allocated_na_set.insert(leases_na[0].toText());
+        allocated_na_vector.push_back(leases_na[0].addr_);
+        allocated_pd_set.insert(leases_pd[0].toText());
+        allocated_pd_vector.push_back(leases_pd[0].addr_);
+    }
+    // Make sure that we have 30 distinct allocations for each lease type.
+    ASSERT_EQ(30, allocated_na_set.size());
+    ASSERT_EQ(30, allocated_na_vector.size());
+    ASSERT_EQ(30, allocated_pd_set.size());
+    ASSERT_EQ(30, allocated_pd_vector.size());
+
+    // Make sure that the addresses are not allocated iteratively.
+    int consecutives = 0;
+    for (auto i = 1; i < allocated_na_vector.size(); ++i) {
+        // Record the cases when the previously allocated address is
+        // lower by 1 (iterative allocation). Some cases like this are
+        // possible even with the random allocation but they should be
+        // very rare.
+        if (IOAddress::increase(allocated_na_vector[i-1]) == allocated_na_vector[i]) {
+            ++consecutives;
+        }
+    }
+    EXPECT_LT(consecutives, 10);
+
+    // Make sure that delegated prefixes have been allocated iteratively.
+    consecutives = 0;
+    for (auto i = 1; i < allocated_pd_vector.size(); ++i) {
+        if (IOAddress::subtract(allocated_pd_vector[i], allocated_pd_vector[i-1]) == IOAddress("0:0:0:1::")) {
+            ++consecutives;
+        }
+    }
+    EXPECT_EQ(29, consecutives);
+}
+
+TEST_F(SARRTest, randomAddressAllocation) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    randomAddressAllocation();
+}
+
+TEST_F(SARRTest, randomAddressAllocationMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    randomAddressAllocation();
+}
+
+void
+SARRTest::randomPrefixAllocation() {
+    // Create the base client and server configuration.
+    Dhcp6Client client;
+    configure(CONFIGS[6], *client.getServer());
+
+    // Record what addresses have been allocated and in what order.
+    std::set<std::string> allocated_na_set;
+    std::vector<IOAddress> allocated_na_vector;
+    std::set<std::string> allocated_pd_set;
+    std::vector<IOAddress> allocated_pd_vector;
+    // Simulate allocations from different clients.
+    for (auto i = 0; i < 30; ++i) {
+        // Create a client from the base client.
+        Dhcp6Client next_client(client.getServer());
+        next_client.requestAddress();
+        next_client.requestPrefix();
+        // Run 4-way exchange.
+        ASSERT_NO_THROW(next_client.doSARR());
+        // We should have one IA_NA and one IA_PD.
+        auto leases_na = next_client.getLeasesByType(Lease::TYPE_NA);
+        ASSERT_EQ(1, leases_na.size());
+        auto leases_pd = next_client.getLeasesByType(Lease::TYPE_PD);
+        ASSERT_EQ(1, leases_pd.size());
+        // Remember allocated address and delegated prefix uniqueness
+        // and order.
+        allocated_na_set.insert(leases_na[0].toText());
+        allocated_na_vector.push_back(leases_na[0].addr_);
+        allocated_pd_set.insert(leases_pd[0].toText());
+        allocated_pd_vector.push_back(leases_pd[0].addr_);
+    }
+    // Make sure that we have 30 distinct allocations for each lease type.
+    ASSERT_EQ(30, allocated_na_set.size());
+    ASSERT_EQ(30, allocated_na_vector.size());
+    ASSERT_EQ(30, allocated_pd_set.size());
+    ASSERT_EQ(30, allocated_pd_vector.size());
+
+    // Make sure that the addresses have been allocated iteratively.
+    int consecutives = 0;
+    for (auto i = 1; i < allocated_na_vector.size(); ++i) {
+        // Record the cases when the previously allocated address is
+        // lower by 1 (iterative allocation).
+        if (IOAddress::increase(allocated_na_vector[i-1]) == allocated_na_vector[i]) {
+            ++consecutives;
+        }
+    }
+
+    // Make sure that addresses have been allocated iteratively.
+    EXPECT_EQ(29, consecutives);
+
+    // Make sure that delegated prefixes have been allocated randomly.
+    consecutives = 0;
+    for (auto i = 1; i < allocated_pd_vector.size(); ++i) {
+        if (IOAddress::subtract(allocated_pd_vector[i], allocated_pd_vector[i-1]) == IOAddress("0:0:0:1::")) {
+            ++consecutives;
+        }
+    }
+    EXPECT_LT(consecutives, 10);
+}
+
+TEST_F(SARRTest, randomPrefixAllocation) {
+    Dhcpv6SrvMTTestGuard guard(*this, false);
+    randomPrefixAllocation();
+}
+
+TEST_F(SARRTest, randomPrefixAllocationMultiThreading) {
+    Dhcpv6SrvMTTestGuard guard(*this, true);
+    randomPrefixAllocation();
+}
+
+
 } // end of anonymous namespace
