@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -17,6 +17,7 @@
 #include <dhcp/hwaddr.h>
 #include <dhcp/option.h>
 #include <dhcp/option_int.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <hooks/hooks.h>
 #include <hooks/hooks_manager.h>
 #include <util/range_utilities.h>
@@ -154,6 +155,7 @@ HATest::createValidJsonConfiguration(const HAConfig::HAMode& ha_mode) const {
         "         \"max-response-delay\": 1000,"
         "         \"max-ack-delay\": 10000,"
         "         \"max-unacked-clients\": 10,"
+        "         \"max-rejected-clients\": 10,"
         "         \"wait-backup-ack\": false,"
         "         \"peers\": ["
         "             {"
@@ -339,6 +341,32 @@ HATest::createQuery6(const std::string& duid_text) const {
     return (query6);
 }
 
+void
+HATest::setDHCPMultiThreadingConfig(bool enabled, uint32_t thread_pool_size,
+                                    uint32_t packet_queue_size) {
+    ElementPtr mt_config = Element::createMap();
+    mt_config->set("enable-multi-threading", Element::create(enabled));
+    mt_config->set("thread-pool-size",
+                   Element::create(static_cast<int>(thread_pool_size)));
+    mt_config->set("queue-size",
+                   Element::create(static_cast<int>(packet_queue_size)));
+    CfgMgr::instance().getStagingCfg()->setDHCPMultiThreading(mt_config);
+}
+
+std::string
+HATest::makeHAMtJson(bool enable_multi_threading, bool http_dedicated_listener,
+                     uint32_t http_listener_threads,  uint32_t http_client_threads) {
+    std::stringstream ss;
+    ss << "\"multi-threading\": {"
+       << " \"enable-multi-threading\": "
+       << (enable_multi_threading ? "true" : "false") << ","
+       << " \"http-dedicated-listener\": "
+       << (http_dedicated_listener ? "true" : "false")  << ","
+       << " \"http-listener-threads\": " << http_listener_threads  << ","
+       << " \"http-client-threads\": " << http_client_threads << "}";
+    return (ss.str());
+}
+
 } // end of namespace isc::ha::test
 } // end of namespace isc::ha
 } // end of namespace isc
