@@ -653,6 +653,23 @@ public:
                                const bool remove_lease,
                                const uint16_t max_unwarned_cycles = 0);
 
+    /// @brief Body of reclaimExpiredLeases6.
+    ///
+    /// @param max_leases Maximum number of leases to be reclaimed.
+    /// @param timeout Maximum amount of time that the reclamation routine
+    /// may be processing expired leases, expressed in milliseconds.
+    /// @param remove_lease A boolean value indicating if the lease should
+    /// be removed when it is reclaimed (if true) or it should be left in the
+    /// database in the "expired-reclaimed" state (if false).
+    /// @param max_unwarned_cycles A number of consecutive processing cycles
+    /// of expired leases, after which the system issues a warning if there
+    /// are still expired leases in the database. If this value is 0, the
+    /// warning is never issued.
+    void reclaimExpiredLeases6Internal(const size_t max_leases,
+                                       const uint16_t timeout,
+                                       const bool remove_lease,
+                                       const uint16_t max_unwarned_cycles = 0);
+
     /// @brief Deletes reclaimed leases expired more than specified amount
     /// of time ago.
     ///
@@ -712,6 +729,23 @@ public:
                                const bool remove_lease,
                                const uint16_t max_unwarned_cycles = 0);
 
+    /// @brief Body of reclaimExpiredLeases4.
+    ///
+    /// @param max_leases Maximum number of leases to be reclaimed.
+    /// @param timeout Maximum amount of time that the reclamation routine
+    /// may be processing expired leases, expressed in milliseconds.
+    /// @param remove_lease A boolean value indicating if the lease should
+    /// be removed when it is reclaimed (if true) or it should be left in the
+    /// database in the "expired-reclaimed" state (if false).
+    /// @param max_unwarned_cycles A number of consecutive processing cycles
+    /// of expired leases, after which the system issues a warning if there
+    /// are still expired leases in the database. If this value is 0, the
+    /// warning is never issued.
+    void reclaimExpiredLeases4Internal(const size_t max_leases,
+                                       const uint16_t timeout,
+                                       const bool remove_lease,
+                                       const uint16_t max_unwarned_cycles = 0);
+
     /// @brief Deletes reclaimed leases expired more than specified amount
     /// of time ago.
     ///
@@ -1710,7 +1744,9 @@ protected:
     /// @param [out] lease A pointer to the lease to be updated.
     /// @param ctx A context containing information from the server about the
     /// client and its message.
-    void updateLease4ExtendedInfo(const Lease4Ptr& lease,
+    /// @return True if there was a significant (e.g. other than cltt) change,
+    /// false otherwise.
+    bool updateLease4ExtendedInfo(const Lease4Ptr& lease,
                                   const ClientContext4& ctx) const;
 
     /// @brief Stores additional client query parameters on a V6 lease
@@ -1731,6 +1767,16 @@ protected:
     void updateLease6ExtendedInfo(const Lease6Ptr& lease,
                                   const ClientContext6& ctx) const;
 
+    /// @brief Clear extended info from a reclaimed V4 lease
+    ///
+    /// @param [out] lease A pointer to the reclaimed lease.
+    void clearReclaimedExtendedInfo(const Lease4Ptr& lease) const;
+
+    /// @brief Clear extended info from a reclaimed V6 lease
+    ///
+    /// @param [out] lease A pointer to the reclaimed lease.
+    void clearReclaimedExtendedInfo(const Lease6Ptr& lease) const;
+
 private:
 
     /// @brief Try to reuse an already allocated lease.
@@ -1790,6 +1836,16 @@ public:
 
     /// @brief The read-write mutex.
     isc::util::ReadWriteMutex rw_mutex_;
+
+    /// @brief Generates a label for subnet or shared-network from subnet
+    ///
+    /// Creates a string for the subnet and its ID for stand alone subnets
+    /// or the shared-network and its name if the given subnet belongs to a
+    /// shared-network.
+    ///
+    /// @param subnet pointer to the source subnet
+    /// @return string contaning the generated label
+    static std::string labelNetworkOrSubnet(SubnetPtr subnet);
 };
 
 /// @brief A pointer to the @c AllocEngine object.
