@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,6 +9,7 @@
 
 #include <cc/data.h>
 #include <cc/simple_parser.h>
+#include <dhcpsrv/cfg_globals.h>
 #include <dhcpsrv/network.h>
 
 namespace isc {
@@ -27,6 +28,14 @@ public:
     /// and a flag are specified.
     static void moveReservationMode(isc::data::ElementPtr config);
 
+    /// @brief Moves deprecated reservation-mode parameter to
+    /// new reservations flags.
+    ///
+    /// @param config [in/out] global parameters to alter.
+    /// @throw DhcpConfigError on error e.g. when both reservation-mode
+    /// and a flag are specified.
+    static void moveReservationMode(CfgGlobalsPtr config);
+
 protected:
 
     /// @brief Parses common parameters
@@ -78,7 +87,7 @@ protected:
     /// @throw DhcpConfigError if configuration of these parameters is
     /// invalid.
     void parseCacheParams(const data::ConstElementPtr& network_data,
-                     NetworkPtr& network);
+                          NetworkPtr& network);
 
     /// @brief Parses parameters pertaining to DDNS behavior.
     ///
@@ -96,6 +105,31 @@ protected:
     /// to be stored.
     void parseDdnsParams(const data::ConstElementPtr& network_data,
                          NetworkPtr& network);
+
+    /// @brief Parses parameters pertaining to allocator selection.
+    ///
+    /// The parsed parameters are:
+    /// - allocator
+    ///
+    /// @param network_data Data element holding shared network
+    /// configuration to be parsed.
+    /// @param [out] network Pointer to a network in which parsed data is
+    /// to be stored.
+    void parseAllocatorParams(const data::ConstElementPtr& network_data,
+                              NetworkPtr& network);
+
+    /// @brief Parses parameters pertaining to prefix delegation allocator
+    /// selection.
+    ///
+    /// The parsed parameters are:
+    /// - pd-allocator
+    ///
+    /// @param network_data Data element holding shared network
+    /// configuration to be parsed.
+    /// @param [out] network Pointer to a network in which parsed data is
+    /// to be stored.
+    void parsePdAllocatorParams(const data::ConstElementPtr& network_data,
+                                Network6Ptr& network);
 };
 
 } // end of namespace isc::dhcp
