@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,12 +10,12 @@
 #include <cc/data.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/cfg_option.h>
-#include <dhcpsrv/parsers/dhcp_parsers.h>
 #include <dhcpsrv/parsers/option_data_parser.h>
 #include <dhcpsrv/parsers/shared_network_parser.h>
 #include <dhcpsrv/parsers/simple_parser4.h>
 #include <dhcpsrv/parsers/simple_parser6.h>
 #include <dhcpsrv/shared_network.h>
+#include <boost/make_shared.hpp>
 #include <boost/pointer_cast.hpp>
 #include <string>
 
@@ -74,17 +74,17 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
             auto json = shared_network_data->get("option-data");
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
-            OptionDataListParser parser(AF_INET);
-            parser.parse(cfg_option, json);
+            auto parser = createOptionDataListParser();
+            parser->parse(cfg_option, json);
         }
 
         if (shared_network_data->contains("subnet4")) {
             auto json = shared_network_data->get("subnet4");
 
             // Create parser instance of subnet4.
-            Subnets4ListConfigParser parser(check_iface_);
+            auto parser = createSubnetsListParser();
             Subnet4Collection subnets;
-            parser.parse(subnets, json);
+            parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
             for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
@@ -197,6 +197,10 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
 
         // Parse lease cache parameters
         parseCacheParams(shared_network_data, network);
+
+        // Parse allocator params.
+        parseAllocatorParams(shared_network_data, network);
+
     } catch (const DhcpConfigError&) {
         // Position was already added
         throw;
@@ -208,13 +212,25 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
     // In order to take advantage of the dynamic inheritance of global
     // parameters to a shared network we need to set a callback function
     // for each shared network to allow for fetching global parameters.
-     shared_network->setFetchGlobalsFn([]() -> ConstElementPtr {
+     shared_network->setFetchGlobalsFn([]() -> ConstCfgGlobalsPtr {
         return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
     });
 
     return (shared_network);
 }
 
+boost::shared_ptr<OptionDataListParser>
+SharedNetwork4Parser::createOptionDataListParser() const {
+    auto parser = boost::make_shared<OptionDataListParser>(AF_INET);
+    return (parser);
+}
+
+boost::shared_ptr<Subnets4ListConfigParser>
+SharedNetwork4Parser::createSubnetsListParser() const {
+    auto parser = boost::make_shared<Subnets4ListConfigParser>(check_iface_);
+    return (parser);
+}
+
 SharedNetwork6Parser::SharedNetwork6Parser(bool check_iface)
     : check_iface_(check_iface) {
 }
@@ -303,8 +319,8 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             auto json = shared_network_data->get("option-data");
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
-            OptionDataListParser parser(AF_INET6);
-            parser.parse(cfg_option, json);
+            auto parser = createOptionDataListParser();
+            parser->parse(cfg_option, json);
         }
 
         if (shared_network_data->contains("client-class")) {
@@ -337,9 +353,9 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             auto json = shared_network_data->get("subnet6");
 
             // Create parser instance of subnet6.
-            Subnets6ListConfigParser parser(check_iface_);
+            auto parser = createSubnetsListParser();
             Subnet6Collection subnets;
-            parser.parse(subnets, json);
+            parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
             for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
@@ -365,6 +381,14 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
 
         // Parse lease cache parameters
         parseCacheParams(shared_network_data, network);
+
+        // Parse allocator params.
+        parseAllocatorParams(shared_network_data, network);
+
+        // Parse prefix delegation allocator params.
+        auto network6 = boost::dynamic_pointer_cast<Network6>(shared_network);
+        parsePdAllocatorParams(shared_network_data, network6);
+
     } catch (const std::exception& ex) {
         isc_throw(DhcpConfigError, ex.what() << " ("
                   << shared_network_data->getPosition() << ")");
@@ -373,12 +397,24 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
     // In order to take advantage of the dynamic inheritance of global
     // parameters to a shared network we need to set a callback function
     // for each shared network which can be used to fetch global parameters.
-     shared_network->setFetchGlobalsFn([]() -> ConstElementPtr {
+     shared_network->setFetchGlobalsFn([]() -> ConstCfgGlobalsPtr {
         return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
     });
 
     return (shared_network);
 }
 
+boost::shared_ptr<OptionDataListParser>
+SharedNetwork6Parser::createOptionDataListParser() const {
+    auto parser = boost::make_shared<OptionDataListParser>(AF_INET6);
+    return (parser);
+}
+
+boost::shared_ptr<Subnets6ListConfigParser>
+SharedNetwork6Parser::createSubnetsListParser() const {
+    auto parser = boost::make_shared<Subnets6ListConfigParser>(check_iface_);
+    return (parser);
+}
+
 } // end of namespace isc::dhcp
 } // end of namespace isc
