@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,12 +6,14 @@
 
 #include <config.h>
 #include <dhcpsrv/tests/alloc_engine_utils.h>
-#include <dhcpsrv/tests/test_utils.h>
+#include <dhcpsrv/testutils/test_utils.h>
 
 #include <hooks/server_hooks.h>
 #include <hooks/callout_manager.h>
 #include <hooks/hooks_manager.h>
 
+#include <iostream>
+
 using namespace std;
 using namespace isc::hooks;
 using namespace isc::asiolink;
@@ -31,8 +33,13 @@ public:
     }
 
     virtual ~HookAllocEngine6Test() {
+        resetCalloutBuffers();
         HooksManager::preCalloutsLibraryHandle().deregisterAllCallouts(
             "lease6_select");
+        bool status = HooksManager::unloadLibraries();
+        if (!status) {
+            cerr << "(fixture dtor) unloadLibraries failed" << endl;
+        }
     }
 
     /// @brief clears out buffers, so callouts can store received arguments
@@ -49,6 +56,21 @@ public:
         callback_skip_ = 0;
     }
 
+    /// @brief Checks if the state of the callout handle associated with a query
+    /// was reset after the callout invocation.
+    ///
+    /// The check includes verification if the status was set to 'continue' and
+    /// that all arguments were deleted.
+    ///
+    /// @param query pointer to the query which callout handle is associated
+    /// with.
+    void checkCalloutHandleReset(const Pkt6Ptr& query) {
+        CalloutHandlePtr callout_handle = query->getCalloutHandle();
+        ASSERT_TRUE(callout_handle);
+        EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
+        EXPECT_TRUE(callout_handle->getArgumentNames().empty());
+    }
+
     /// callback that stores received callout name and received values
     static int
     lease6_select_callout(CalloutHandle& callout_handle) {
@@ -84,8 +106,6 @@ public:
         callout_handle.getArgument("lease6", lease);
         callback_addr_updated_ = addr_override_;
         lease->addr_ = callback_addr_updated_;
-        lease->t1_ = t1_override_;
-        lease->t2_ = t2_override_;
         lease->preferred_lft_ = pref_override_;
         lease->valid_lft_ = valid_override_;
 
@@ -109,8 +129,6 @@ public:
 
     // Values to be used in callout to override lease6 content
     static const IOAddress addr_override_;
-    static const uint32_t t1_override_;
-    static const uint32_t t2_override_;
     static const uint32_t pref_override_;
     static const uint32_t valid_override_;
 
@@ -135,8 +153,6 @@ public:
 // linker complains about undefined references if they are defined within
 // the class declaration.
 const IOAddress HookAllocEngine6Test::addr_override_("2001:db8::abcd");
-const uint32_t HookAllocEngine6Test::t1_override_ = 6000;
-const uint32_t HookAllocEngine6Test::t2_override_ = 7000;
 const uint32_t HookAllocEngine6Test::pref_override_ = 8000;
 const uint32_t HookAllocEngine6Test::valid_override_ = 9000;
 
@@ -167,12 +183,12 @@ TEST_F(HookAllocEngine6Test, lease6_select) {
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE, 100)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install lease6_select
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -227,6 +243,9 @@ TEST_F(HookAllocEngine6Test, lease6_select) {
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
 
     EXPECT_TRUE(callback_qry_options_copy_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
 // This test checks if lease6_select callout is able to override the values
@@ -235,20 +254,18 @@ TEST_F(HookAllocEngine6Test, change_lease6_select) {
 
     // Make sure that the overridden values are different than the ones from
     // subnet originally used to create the lease
-    ASSERT_NE(t1_override_, subnet_->getT1());
-    ASSERT_NE(t2_override_, subnet_->getT2());
     ASSERT_NE(pref_override_, subnet_->getPreferred());
     ASSERT_NE(valid_override_, subnet_->getValid());
     ASSERT_FALSE(subnet_->inRange(addr_override_));
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE, 100)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install a callout
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -266,8 +283,6 @@ TEST_F(HookAllocEngine6Test, change_lease6_select) {
 
     // See if the values overridden by callout are there
     EXPECT_TRUE(lease->addr_.equals(addr_override_));
-    EXPECT_EQ(t1_override_, lease->t1_);
-    EXPECT_EQ(t2_override_, lease->t2_);
     EXPECT_EQ(pref_override_, lease->preferred_lft_);
     EXPECT_EQ(valid_override_, lease->valid_lft_);
 
@@ -278,10 +293,11 @@ TEST_F(HookAllocEngine6Test, change_lease6_select) {
 
     // Check if values in the database are overridden
     EXPECT_TRUE(from_mgr->addr_.equals(addr_override_));
-    EXPECT_EQ(t1_override_, from_mgr->t1_);
-    EXPECT_EQ(t2_override_, from_mgr->t2_);
     EXPECT_EQ(pref_override_, from_mgr->preferred_lft_);
     EXPECT_EQ(valid_override_, from_mgr->valid_lft_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
 // This test checks if lease6_select callout can set the status to next
@@ -290,12 +306,12 @@ TEST_F(HookAllocEngine6Test, skip_lease6_select) {
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE, 100)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install a callout
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -313,6 +329,9 @@ TEST_F(HookAllocEngine6Test, skip_lease6_select) {
 
     // Check no retry was attempted
     EXPECT_EQ(1, callback_skip_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
 /// @brief helper class used in Hooks testing in AllocEngine4
@@ -326,12 +345,19 @@ TEST_F(HookAllocEngine6Test, skip_lease6_select) {
 class HookAllocEngine4Test : public AllocEngine4Test {
 public:
     HookAllocEngine4Test() {
+        // The default context is not used in these tests.
+        ctx_.callout_handle_.reset();
         resetCalloutBuffers();
     }
 
     virtual ~HookAllocEngine4Test() {
+        resetCalloutBuffers();
         HooksManager::preCalloutsLibraryHandle().deregisterAllCallouts(
             "lease4_select");
+        bool status = HooksManager::unloadLibraries();
+        if (!status) {
+            cerr << "(fixture dtor) unloadLibraries failed" << endl;
+        }
     }
 
     /// @brief clears out buffers, so callouts can store received arguments
@@ -348,6 +374,21 @@ public:
         callback_skip_ = 0;
     }
 
+    /// @brief Checks if the state of the callout handle associated with a query
+    /// was reset after the callout invocation.
+    ///
+    /// The check includes verification if the status was set to 'continue' and
+    /// that all arguments were deleted.
+    ///
+    /// @param query pointer to the query which callout handle is associated
+    /// with.
+    void checkCalloutHandleReset(const Pkt4Ptr& query) {
+        CalloutHandlePtr callout_handle = query->getCalloutHandle();
+        ASSERT_TRUE(callout_handle);
+        EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
+        EXPECT_TRUE(callout_handle->getArgumentNames().empty());
+    }
+
     /// callback that stores received callout name and received values
     static int
     lease4_select_callout(CalloutHandle& callout_handle) {
@@ -383,8 +424,6 @@ public:
         callout_handle.getArgument("lease4", lease);
         callback_addr_updated_ = addr_override_;
         lease->addr_ = callback_addr_updated_;
-        lease->t1_ = t1_override_;
-        lease->t2_ = t2_override_;
         lease->valid_lft_ = valid_override_;
 
         return (0);
@@ -407,8 +446,6 @@ public:
 
     // Values to be used in callout to override lease4 content
     static const IOAddress addr_override_;
-    static const uint32_t t1_override_;
-    static const uint32_t t2_override_;
     static const uint32_t valid_override_;
 
     // Callback will store original and overridden values here
@@ -432,8 +469,6 @@ public:
 // linker complains about undefined references if they are defined within
 // the class declaration.
 const IOAddress HookAllocEngine4Test::addr_override_("192.0.3.1");
-const uint32_t HookAllocEngine4Test::t1_override_ = 4000;
-const uint32_t HookAllocEngine4Test::t2_override_ = 7000;
 const uint32_t HookAllocEngine4Test::valid_override_ = 9000;
 
 IOAddress HookAllocEngine4Test::callback_addr_original_("::");
@@ -463,13 +498,12 @@ TEST_F(HookAllocEngine4Test, lease4_select) {
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE,
-                                                 100, false)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install lease4_select
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -522,6 +556,9 @@ TEST_F(HookAllocEngine4Test, lease4_select) {
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
 
     EXPECT_TRUE(callback_qry_options_copy_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
 // This test checks if lease4_select callout is able to override the values
@@ -530,20 +567,17 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
 
     // Make sure that the overridden values are different than the ones from
     // subnet originally used to create the lease
-    ASSERT_NE(t1_override_, subnet_->getT1());
-    ASSERT_NE(t2_override_, subnet_->getT2());
     ASSERT_NE(valid_override_, subnet_->getValid());
     ASSERT_FALSE(subnet_->inRange(addr_override_));
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE,
-                                                 100, false)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install a callout
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -567,8 +601,6 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
 
     // See if the values overridden by callout are there
     EXPECT_TRUE(lease->addr_.equals(addr_override_));
-    EXPECT_EQ(t1_override_, lease->t1_);
-    EXPECT_EQ(t2_override_, lease->t2_);
     EXPECT_EQ(valid_override_, lease->valid_lft_);
 
     // Now check if the lease is in the database
@@ -577,9 +609,10 @@ TEST_F(HookAllocEngine4Test, change_lease4_select) {
 
     // Check if values in the database are overridden
     EXPECT_TRUE(from_mgr->addr_.equals(addr_override_));
-    EXPECT_EQ(t1_override_, from_mgr->t1_);
-    EXPECT_EQ(t2_override_, from_mgr->t2_);
     EXPECT_EQ(valid_override_, from_mgr->valid_lft_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
 // This test checks if lease4_select callout can set the status to next
@@ -588,13 +621,12 @@ TEST_F(HookAllocEngine4Test, skip_lease4_select) {
 
     // Create allocation engine (hook names are registered in its ctor)
     boost::scoped_ptr<AllocEngine> engine;
-    ASSERT_NO_THROW(engine.reset(new AllocEngine(AllocEngine::ALLOC_ITERATIVE,
-                                                 100, false)));
+    ASSERT_NO_THROW(engine.reset(new AllocEngine(100)));
     ASSERT_TRUE(engine);
 
     // Initialize Hooks Manager
     HookLibsCollection libraries; // no libraries at this time
-    HooksManager::loadLibraries(libraries);
+    ASSERT_NO_THROW(HooksManager::loadLibraries(libraries));
 
     // Install a callout
     EXPECT_NO_THROW(HooksManager::preCalloutsLibraryHandle().registerCallout(
@@ -615,8 +647,11 @@ TEST_F(HookAllocEngine4Test, skip_lease4_select) {
 
     // Check no retry was attempted
     EXPECT_EQ(1, callback_skip_);
+
+    // Check if the callout handle state was reset after the callout.
+    checkCalloutHandleReset(ctx.query_);
 }
 
-}; // namespace test
-}; // namespace dhcp
-}; // namespace isc
+} // namespace test
+} // namespace dhcp
+} // namespace isc
