@@ -1,4 +1,4 @@
-// Copyright (C) 2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -191,22 +191,22 @@ public:
 };
 
 /// @brief Test fixture class for testing threading modes of HTTP client.
-class MtHttpClientTest : public ::testing::Test {
+class MultiThreadingHttpClientTest : public ::testing::Test {
 public:
 
     /// @brief Constructor.
-    MtHttpClientTest()
+    MultiThreadingHttpClientTest()
         : io_service_(), client_(), listener_(), factory_(), listeners_(), factories_(),
           test_timer_(io_service_), num_threads_(0), num_batches_(0), num_listeners_(0),
           expected_requests_(0), num_in_progress_(0), num_finished_(0), paused_(false),
           pause_cnt_(0) {
-        test_timer_.setup(std::bind(&MtHttpClientTest::timeoutHandler, this, true),
+        test_timer_.setup(std::bind(&MultiThreadingHttpClientTest::timeoutHandler, this, true),
                           TEST_TIMEOUT, IntervalTimer::ONE_SHOT);
         MultiThreadingMgr::instance().setMode(true);
     }
 
     /// @brief Destructor.
-    ~MtHttpClientTest() {
+    ~MultiThreadingHttpClientTest() {
         // Stop the client.
         if (client_) {
             client_->stop();
@@ -271,6 +271,14 @@ public:
         return (request);
     }
 
+    /// @brief Test that worker threads are not permitted to change thread pool
+    /// state.
+    void testIllegalThreadPoolActions() {
+        ASSERT_THROW(client_->start(), MultiThreadingInvalidOperation);
+        ASSERT_THROW(client_->pause(), MultiThreadingInvalidOperation);
+        ASSERT_THROW(client_->resume(), MultiThreadingInvalidOperation);
+    }
+
     /// @brief Initiates a single HTTP request.
     ///
     /// Constructs an HTTP post whose body is a JSON map containing a
@@ -320,6 +328,12 @@ public:
                 }
             }
 
+            // If running on multiple threads, threads should be prohibited from
+            // changing the thread pool state.
+            if (num_threads_) {
+                testIllegalThreadPoolActions();
+            }
+
             // Get stringified thread-id.
             std::stringstream ss;
             ss << std::this_thread::get_id();
@@ -467,7 +481,7 @@ public:
         }
 
         // Create an MT client with num_threads
-        ASSERT_NO_THROW_LOG(client_.reset(new HttpClient(io_service_, num_threads)));
+        ASSERT_NO_THROW_LOG(client_.reset(new HttpClient(io_service_, num_threads, true)));
         ASSERT_TRUE(client_);
 
         if (num_threads_ == 0) {
@@ -478,10 +492,6 @@ public:
             ASSERT_TRUE(client_->getThreadIOService());
         }
 
-        // Verify the pool size and number of threads are as expected.
-        ASSERT_EQ(client_->getThreadPoolSize(), num_threads);
-        ASSERT_EQ(client_->getThreadCount(), num_threads);
-
         // Start the requisite number of requests:
         //   batch * listeners * threads.
         int sequence = 0;
@@ -493,6 +503,12 @@ public:
             }
         }
 
+        client_->start();
+
+        // Verify the pool size and number of threads are as expected.
+        ASSERT_EQ(client_->getThreadPoolSize(), num_threads);
+        ASSERT_EQ(client_->getThreadCount(), num_threads);
+
         // Loop until the clients are done, an error occurs, or the time runs out.
         runIOService(expected_requests_);
 
@@ -631,18 +647,9 @@ public:
         }
 
         // Create an instant start, MT client with num_threads
-        ASSERT_NO_THROW_LOG(client_.reset(new HttpClient(io_service_, num_threads)));
+        ASSERT_NO_THROW_LOG(client_.reset(new HttpClient(io_service_, num_threads, true)));
         ASSERT_TRUE(client_);
 
-        // Client should be running. Check convenience functions.
-        ASSERT_TRUE(client_->isRunning());
-        ASSERT_FALSE(client_->isPaused());
-        ASSERT_FALSE(client_->isStopped());
-
-        // Verify the pool size and number of threads are as expected.
-        ASSERT_EQ(client_->getThreadPoolSize(), num_threads);
-        ASSERT_EQ(client_->getThreadCount(), num_threads);
-
         // Start the requisite number of requests:
         //   batch * listeners * threads.
         int sequence = 0;
@@ -654,6 +661,17 @@ public:
             }
         }
 
+        client_->start();
+
+        // Client should be running. Check convenience functions.
+        ASSERT_TRUE(client_->isRunning());
+        ASSERT_FALSE(client_->isPaused());
+        ASSERT_FALSE(client_->isStopped());
+
+        // Verify the pool size and number of threads are as expected.
+        ASSERT_EQ(client_->getThreadPoolSize(), num_threads);
+        ASSERT_EQ(client_->getThreadCount(), num_threads);
+
         size_t rr_count = 0;
         while (rr_count < total_requests) {
             size_t request_limit = (pause_cnt_ < num_pauses ?
@@ -811,7 +829,7 @@ public:
 
 // Verifies we can construct and destruct, in both single
 // and multi-threaded modes.
-TEST_F(MtHttpClientTest, basics) {
+TEST_F(MultiThreadingHttpClientTest, basics) {
     MultiThreadingMgr::instance().setMode(false);
     HttpClientPtr client;
 
@@ -879,7 +897,7 @@ TEST_F(MtHttpClientTest, basics) {
 }
 
 // Verifies we can construct with deferred start.
-TEST_F(MtHttpClientTest, deferredStart) {
+TEST_F(MultiThreadingHttpClientTest, deferredStart) {
     MultiThreadingMgr::instance().setMode(true);
     HttpClientPtr client;
     size_t thread_pool_size = 3;
@@ -923,7 +941,7 @@ TEST_F(MtHttpClientTest, deferredStart) {
 }
 
 // Verifies we can restart after stop.
-TEST_F(MtHttpClientTest, restartAfterStop) {
+TEST_F(MultiThreadingHttpClientTest, restartAfterStop) {
     MultiThreadingMgr::instance().setMode(true);
     HttpClientPtr client;
     size_t thread_pool_size = 3;
@@ -964,14 +982,14 @@ TEST_F(MtHttpClientTest, restartAfterStop) {
 // requests, and listeners.
 
 // Single-threaded, three batches, one listener.
-TEST_F(MtHttpClientTest, zeroByThreeByOne) {
+TEST_F(MultiThreadingHttpClientTest, zeroByThreeByOne) {
     size_t num_threads = 0; // Zero threads = ST mode.
     size_t num_batches = 3;
     threadRequestAndReceive(num_threads, num_batches);
 }
 
 // Single-threaded, three batches, three listeners.
-TEST_F(MtHttpClientTest, zeroByThreeByThree) {
+TEST_F(MultiThreadingHttpClientTest, zeroByThreeByThree) {
     size_t num_threads = 0; // Zero threads = ST mode.
     size_t num_batches = 3;
     size_t num_listeners = 3;
@@ -979,28 +997,28 @@ TEST_F(MtHttpClientTest, zeroByThreeByThree) {
 }
 
 // Multi-threaded with one thread, three batches, one listener
-TEST_F(MtHttpClientTest, oneByThreeByOne) {
+TEST_F(MultiThreadingHttpClientTest, oneByThreeByOne) {
     size_t num_threads = 1;
     size_t num_batches = 3;
     threadRequestAndReceive(num_threads, num_batches);
 }
 
 // Multi-threaded with three threads, three batches, one listener
-TEST_F(MtHttpClientTest, threeByThreeByOne) {
+TEST_F(MultiThreadingHttpClientTest, threeByThreeByOne) {
     size_t num_threads = 3;
     size_t num_batches = 3;
     threadRequestAndReceive(num_threads, num_batches);
 }
 
 // Multi-threaded with three threads, nine batches, one listener
-TEST_F(MtHttpClientTest, threeByNineByOne) {
+TEST_F(MultiThreadingHttpClientTest, threeByNineByOne) {
     size_t num_threads = 3;
     size_t num_batches = 9;
     threadRequestAndReceive(num_threads, num_batches);
 }
 
 // Multi-threaded with two threads, four batches, two listeners
-TEST_F(MtHttpClientTest, twoByFourByTwo) {
+TEST_F(MultiThreadingHttpClientTest, twoByFourByTwo) {
     size_t num_threads = 2;
     size_t num_batches = 4;
     size_t num_listeners = 2;
@@ -1008,7 +1026,7 @@ TEST_F(MtHttpClientTest, twoByFourByTwo) {
 }
 
 // Multi-threaded with four threads, four batches, two listeners
-TEST_F(MtHttpClientTest, fourByFourByTwo) {
+TEST_F(MultiThreadingHttpClientTest, fourByFourByTwo) {
     size_t num_threads = 4;
     size_t num_batches = 4;
     size_t num_listeners = 2;
@@ -1017,7 +1035,7 @@ TEST_F(MtHttpClientTest, fourByFourByTwo) {
 
 // Verifies that we can cleanly pause, resume, and shutdown while doing
 // multi-threaded work.
-TEST_F(MtHttpClientTest, workPauseResumeShutdown) {
+TEST_F(MultiThreadingHttpClientTest, workPauseResumeShutdown) {
     size_t num_threads = 4;
     size_t num_batches = 4;
     size_t num_listeners = 4;
